Package: roxygen2
Title: In-Line Documentation for R
Version: 8.1.0
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Peter", "Danenberg", , "pcd@roxygen.org", role = c("aut", "cph")),
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = "aut"),
    person("Manuel", "Eugster", role = c("aut", "cph")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Generate your Rd documentation, 'NAMESPACE' file, and
    collation field using specially formatted comments. Writing
    documentation in-line with code makes it easier to keep your
    documentation up-to-date as your requirements change. 'roxygen2' is
    inspired by the 'Doxygen' system for C++.
License: MIT + file LICENSE
URL: https://roxygen2.r-lib.org/, https://github.com/r-lib/roxygen2
BugReports: https://github.com/r-lib/roxygen2/issues
Depends: R (>= 4.1)
Imports: brew, cli (>= 3.3.0), commonmark, desc (>= 1.2.0), knitr,
        lifecycle, methods, pkgload (>= 1.5.2), R6 (>= 2.1.2), rdtools
        (>= 0.1.0), rlang (>= 1.1.0), utils, withr, xml2
Suggests: covr, R.methodsS3, S7, R.oo, rmarkdown (>= 2.16), testthat
        (>= 3.1.2), yaml
LinkingTo: cpp11
VignetteBuilder: knitr
Config/Needs/development: testthat
Config/Needs/website: tidyverse/tidytemplate
Config/roxygen2/load: installed
Config/roxygen2/markdown: TRUE
Config/roxygen2/version: 8.0.0.9000
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Encoding: UTF-8
Language: en-GB
NeedsCompilation: yes
Packaged: 2026-07-31 13:20:14 UTC; hadleywickham
Author: Hadley Wickham [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-4757-117X>),
  Peter Danenberg [aut, cph],
  Gábor Csárdi [aut],
  Manuel Eugster [aut, cph],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-08-04 20:00:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-08-04 23:19:46 UTC; unix
Archs: roxygen2.so.dSYM
