Package: testCollateMissing
Title: Tools to make developing R code easier
License: GPL-2
Description: 
Author: Hadley <h.wickham@gmail.com>
Maintainer: Hadley <h.wickham@gmail.com>
Version: 0.1
Collate:
    'b.r'
