Package: testCollateNoIncludes
Title: Test no change to Collate when there are no @includes
License: GPL-2
Description:
Author: Geoff <geoff.lee99@gmail.com>
Maintainer: Geoff <geoff.lee99@gmail.com>
Version: 0.1
Collate: b.R a.R
