# Generated by roxygen2: do not edit by hand

S3method(block_to_rd,default)
S3method(block_to_rd,roxy_block)
S3method(block_to_rd,roxy_block_r6class)
S3method(c,rd)
S3method(default_export,"NULL")
S3method(default_export,default)
S3method(default_export,rcclass)
S3method(default_export,s3method)
S3method(default_export,s4class)
S3method(default_export,s4generic)
S3method(default_export,s4method)
S3method(escape,"NULL")
S3method(escape,character)
S3method(escape,rd)
S3method(format,object)
S3method(format,rd_r6_class)
S3method(format,rd_r6_field)
S3method(format,rd_r6_fields)
S3method(format,rd_r6_inherited)
S3method(format,rd_r6_method)
S3method(format,rd_r6_methods)
S3method(format,rd_r6_super)
S3method(format,rd_section)
S3method(format,rd_section_alias)
S3method(format,rd_section_author)
S3method(format,rd_section_backref)
S3method(format,rd_section_concept)
S3method(format,rd_section_description)
S3method(format,rd_section_details)
S3method(format,rd_section_docType)
S3method(format,rd_section_encoding)
S3method(format,rd_section_examples)
S3method(format,rd_section_family)
S3method(format,rd_section_field)
S3method(format,rd_section_formals)
S3method(format,rd_section_format)
S3method(format,rd_section_inherit)
S3method(format,rd_section_inherit_dot_params)
S3method(format,rd_section_inherit_params_args)
S3method(format,rd_section_inherit_section)
S3method(format,rd_section_keyword)
S3method(format,rd_section_minidesc)
S3method(format,rd_section_name)
S3method(format,rd_section_note)
S3method(format,rd_section_package)
S3method(format,rd_section_param)
S3method(format,rd_section_prop)
S3method(format,rd_section_r6_class)
S3method(format,rd_section_rawRd)
S3method(format,rd_section_rcmethods)
S3method(format,rd_section_reexport)
S3method(format,rd_section_references)
S3method(format,rd_section_section)
S3method(format,rd_section_seealso)
S3method(format,rd_section_slot)
S3method(format,rd_section_source)
S3method(format,rd_section_title)
S3method(format,rd_section_usage)
S3method(format,rd_section_value)
S3method(format,roxy_tag)
S3method(merge,rd_section)
S3method(merge,rd_section_author)
S3method(merge,rd_section_inherit)
S3method(merge,rd_section_inherit_dot_params)
S3method(merge,rd_section_inherit_params_args)
S3method(merge,rd_section_inherit_section)
S3method(merge,rd_section_minidesc)
S3method(merge,rd_section_param)
S3method(merge,rd_section_prop)
S3method(merge,rd_section_r6_class)
S3method(merge,rd_section_reexport)
S3method(merge,rd_section_section)
S3method(merge,rd_section_seealso)
S3method(object_defaults,"function")
S3method(object_defaults,data)
S3method(object_defaults,default)
S3method(object_defaults,import)
S3method(object_defaults,package)
S3method(object_defaults,r6class)
S3method(object_defaults,r6method)
S3method(object_defaults,rcclass)
S3method(object_defaults,s3generic)
S3method(object_defaults,s3method)
S3method(object_defaults,s4class)
S3method(object_defaults,s4generic)
S3method(object_defaults,s4method)
S3method(object_defaults,s7class)
S3method(object_defaults,s7generic)
S3method(object_defaults,s7method)
S3method(object_defaults,value)
S3method(object_format,default)
S3method(object_name,"function")
S3method(object_name,default)
S3method(object_name,s3generic)
S3method(object_name,s3method)
S3method(object_name,s4generic)
S3method(object_name,s4method)
S3method(object_usage,"function")
S3method(object_usage,data)
S3method(object_usage,default)
S3method(object_usage,s3method)
S3method(object_usage,s4generic)
S3method(object_usage,s4method)
S3method(object_usage,s7class)
S3method(object_usage,s7generic)
S3method(object_usage,s7method)
S3method(object_usage,value)
S3method(print,object)
S3method(print,rd)
S3method(print,rd_section)
S3method(print,roxy_block)
S3method(print,roxy_tag)
S3method(roclet_clean,roclet_namespace)
S3method(roclet_clean,roclet_rd)
S3method(roclet_output,roclet_namespace)
S3method(roclet_output,roclet_rd)
S3method(roclet_output,roclet_vignette)
S3method(roclet_preprocess,default)
S3method(roclet_process,roclet_namespace)
S3method(roclet_process,roclet_rd)
S3method(roclet_process,roclet_vignette)
S3method(roxy_tag_ns,default)
S3method(roxy_tag_ns,roxy_tag_evalNamespace)
S3method(roxy_tag_ns,roxy_tag_export)
S3method(roxy_tag_ns,roxy_tag_exportClass)
S3method(roxy_tag_ns,roxy_tag_exportMethod)
S3method(roxy_tag_ns,roxy_tag_exportPattern)
S3method(roxy_tag_ns,roxy_tag_exportS3Method)
S3method(roxy_tag_ns,roxy_tag_import)
S3method(roxy_tag_ns,roxy_tag_importClassesFrom)
S3method(roxy_tag_ns,roxy_tag_importFrom)
S3method(roxy_tag_ns,roxy_tag_importMethodsFrom)
S3method(roxy_tag_ns,roxy_tag_rawNamespace)
S3method(roxy_tag_ns,roxy_tag_useDynLib)
S3method(roxy_tag_parse,default)
S3method(roxy_tag_parse,roxy_tag_R6method)
S3method(roxy_tag_parse,roxy_tag_aliases)
S3method(roxy_tag_parse,roxy_tag_author)
S3method(roxy_tag_parse,roxy_tag_backref)
S3method(roxy_tag_parse,roxy_tag_concept)
S3method(roxy_tag_parse,roxy_tag_describeIn)
S3method(roxy_tag_parse,roxy_tag_description)
S3method(roxy_tag_parse,roxy_tag_details)
S3method(roxy_tag_parse,roxy_tag_docType)
S3method(roxy_tag_parse,roxy_tag_encoding)
S3method(roxy_tag_parse,roxy_tag_eval)
S3method(roxy_tag_parse,roxy_tag_evalNamespace)
S3method(roxy_tag_parse,roxy_tag_evalRd)
S3method(roxy_tag_parse,roxy_tag_example)
S3method(roxy_tag_parse,roxy_tag_examples)
S3method(roxy_tag_parse,roxy_tag_examplesIf)
S3method(roxy_tag_parse,roxy_tag_export)
S3method(roxy_tag_parse,roxy_tag_exportClass)
S3method(roxy_tag_parse,roxy_tag_exportMethod)
S3method(roxy_tag_parse,roxy_tag_exportPattern)
S3method(roxy_tag_parse,roxy_tag_exportS3Method)
S3method(roxy_tag_parse,roxy_tag_family)
S3method(roxy_tag_parse,roxy_tag_field)
S3method(roxy_tag_parse,roxy_tag_format)
S3method(roxy_tag_parse,roxy_tag_import)
S3method(roxy_tag_parse,roxy_tag_importClassesFrom)
S3method(roxy_tag_parse,roxy_tag_importFrom)
S3method(roxy_tag_parse,roxy_tag_importMethodsFrom)
S3method(roxy_tag_parse,roxy_tag_include)
S3method(roxy_tag_parse,roxy_tag_includeRmd)
S3method(roxy_tag_parse,roxy_tag_inherit)
S3method(roxy_tag_parse,roxy_tag_inheritDotParams)
S3method(roxy_tag_parse,roxy_tag_inheritParams)
S3method(roxy_tag_parse,roxy_tag_inheritSection)
S3method(roxy_tag_parse,roxy_tag_keywords)
S3method(roxy_tag_parse,roxy_tag_md)
S3method(roxy_tag_parse,roxy_tag_method)
S3method(roxy_tag_parse,roxy_tag_name)
S3method(roxy_tag_parse,roxy_tag_noMd)
S3method(roxy_tag_parse,roxy_tag_noRd)
S3method(roxy_tag_parse,roxy_tag_note)
S3method(roxy_tag_parse,roxy_tag_order)
S3method(roxy_tag_parse,roxy_tag_param)
S3method(roxy_tag_parse,roxy_tag_prop)
S3method(roxy_tag_parse,roxy_tag_rawNamespace)
S3method(roxy_tag_parse,roxy_tag_rawRd)
S3method(roxy_tag_parse,roxy_tag_rdname)
S3method(roxy_tag_parse,roxy_tag_references)
S3method(roxy_tag_parse,roxy_tag_return)
S3method(roxy_tag_parse,roxy_tag_returns)
S3method(roxy_tag_parse,roxy_tag_section)
S3method(roxy_tag_parse,roxy_tag_seealso)
S3method(roxy_tag_parse,roxy_tag_slot)
S3method(roxy_tag_parse,roxy_tag_source)
S3method(roxy_tag_parse,roxy_tag_template)
S3method(roxy_tag_parse,roxy_tag_templateVar)
S3method(roxy_tag_parse,roxy_tag_title)
S3method(roxy_tag_parse,roxy_tag_usage)
S3method(roxy_tag_parse,roxy_tag_useDynLib)
S3method(roxy_tag_rd,default)
S3method(roxy_tag_rd,roxy_tag_.formals)
S3method(roxy_tag_rd,roxy_tag_.methods)
S3method(roxy_tag_rd,roxy_tag_.package)
S3method(roxy_tag_rd,roxy_tag_.reexport)
S3method(roxy_tag_rd,roxy_tag_author)
S3method(roxy_tag_rd,roxy_tag_backref)
S3method(roxy_tag_rd,roxy_tag_concept)
S3method(roxy_tag_rd,roxy_tag_description)
S3method(roxy_tag_rd,roxy_tag_details)
S3method(roxy_tag_rd,roxy_tag_docType)
S3method(roxy_tag_rd,roxy_tag_encoding)
S3method(roxy_tag_rd,roxy_tag_evalRd)
S3method(roxy_tag_rd,roxy_tag_example)
S3method(roxy_tag_rd,roxy_tag_examples)
S3method(roxy_tag_rd,roxy_tag_examplesIf)
S3method(roxy_tag_rd,roxy_tag_family)
S3method(roxy_tag_rd,roxy_tag_field)
S3method(roxy_tag_rd,roxy_tag_format)
S3method(roxy_tag_rd,roxy_tag_includeRmd)
S3method(roxy_tag_rd,roxy_tag_inherit)
S3method(roxy_tag_rd,roxy_tag_inheritDotParams)
S3method(roxy_tag_rd,roxy_tag_inheritParams)
S3method(roxy_tag_rd,roxy_tag_inheritSection)
S3method(roxy_tag_rd,roxy_tag_keywords)
S3method(roxy_tag_rd,roxy_tag_note)
S3method(roxy_tag_rd,roxy_tag_param)
S3method(roxy_tag_rd,roxy_tag_prop)
S3method(roxy_tag_rd,roxy_tag_rawRd)
S3method(roxy_tag_rd,roxy_tag_references)
S3method(roxy_tag_rd,roxy_tag_return)
S3method(roxy_tag_rd,roxy_tag_returns)
S3method(roxy_tag_rd,roxy_tag_section)
S3method(roxy_tag_rd,roxy_tag_seealso)
S3method(roxy_tag_rd,roxy_tag_slot)
S3method(roxy_tag_rd,roxy_tag_source)
S3method(roxy_tag_rd,roxy_tag_title)
S3method(roxy_tag_rd,roxy_tag_usage)
export(block_get_tag)
export(block_get_tag_value)
export(block_get_tags)
export(block_has_tags)
export(env_file)
export(env_package)
export(escape_examples)
export(is_s3_generic)
export(is_s3_method)
export(load_installed)
export(load_options)
export(load_pkgload)
export(load_source)
export(namespace_roclet)
export(needs_roxygenize)
export(object)
export(object_format)
export(parse_file)
export(parse_package)
export(parse_text)
export(rd_roclet)
export(rd_section)
export(roc_proc_text)
export(roclet)
export(roclet_clean)
export(roclet_find)
export(roclet_output)
export(roclet_preprocess)
export(roclet_process)
export(roclet_tags)
export(roxy_block)
export(roxy_meta_get)
export(roxy_tag)
export(roxy_tag_parse)
export(roxy_tag_rd)
export(roxy_tag_warning)
export(roxygenise)
export(roxygenize)
export(tag_code)
export(tag_examples)
export(tag_inherit)
export(tag_markdown)
export(tag_markdown_with_sections)
export(tag_name)
export(tag_name_description)
export(tag_toggle)
export(tag_two_part)
export(tag_value)
export(tag_words)
export(tag_words_line)
export(tags_list)
export(tags_metadata)
export(update_collate)
export(vignette_roclet)
export(warn_roxy_tag)
import(rlang)
importFrom(R6,R6Class)
importFrom(knitr,knit)
importFrom(knitr,opts_chunk)
importFrom(lifecycle,deprecated)
importFrom(stats,setNames)
importFrom(utils,URLdecode)
importFrom(utils,URLencode)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_children)
importFrom(xml2,xml_contents)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_name)
importFrom(xml2,xml_ns_strip)
importFrom(xml2,xml_text)
importFrom(xml2,xml_type)
useDynLib(roxygen2, .registration=TRUE)
