test_that("determine if file is autogenerated", {
  expect_true(made_by_roxygen("made-by-roxygen/with-header.Rd"))
  expect_false(made_by_roxygen("made-by-roxygen/without-header.Rd"))
  expect_false(made_by_roxygen("made-by-roxygen/empty.Rd"))
  expect_true(check_made_by(made_by("#")))
  expect_false(check_made_by(character()))
})
