# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

escapeExamples <- function(x) {
    .Call(`_roxygen2_escapeExamples`, x)
}

findEndOfTag <- function(string, is_code = FALSE) {
    .Call(`_roxygen2_findEndOfTag`, string, is_code)
}

rdComplete <- function(string, is_code = FALSE) {
    .Call(`_roxygen2_rdComplete`, string, is_code)
}

leadingSpaces <- function(lines) {
    .Call(`_roxygen2_leadingSpaces`, lines)
}

tokenise_block <- function(lines, file = "", offset = 0L) {
    .Call(`_roxygen2_tokenise_block`, lines, file, offset)
}

find_includes <- function(path) {
    .Call(`_roxygen2_find_includes`, path)
}

wrapUsage <- function(string, width = 80L, indent = 2L) {
    .Call(`_roxygen2_wrapUsage`, string, width, indent)
}

