#' @useDynLib roxygen2, .registration=TRUE
#' @keywords internal
#' @import rlang
"_PACKAGE"

## usethis namespace: start
#' @importFrom knitr knit
#' @importFrom knitr opts_chunk
#' @importFrom lifecycle deprecated
#' @importFrom R6 R6Class
#' @importFrom stats setNames
#' @importFrom utils head
#' @importFrom utils tail
#' @importFrom utils URLdecode
#' @importFrom utils URLencode
#' @importFrom xml2 xml_attr
#' @importFrom xml2 xml_children
#' @importFrom xml2 xml_contents
#' @importFrom xml2 xml_find_all
#' @importFrom xml2 xml_name
#' @importFrom xml2 xml_ns_strip
#' @importFrom xml2 xml_text
#' @importFrom xml2 xml_type
## usethis namespace: end
NULL
