#' @details
#' See `vignette("roxygen2", package = "roxygen2")` for an overview
#' of the package, `vignette("rd", package = "roxygen2")` for generating
#' documentation, and `vignette("namespace", package = "roxygen2")` for
#' generating the namespace specification.
#'
#' If you have existing Rd files, check out the `Rd2roxygen` package
#' for a convenient way of converting Rd files to roxygen comments.
#'
#' @useDynLib roxygen2, .registration=TRUE
#' @keywords internal
#' @importFrom Rcpp sourceCpp
#' @importFrom R6 R6Class
#' @importFrom purrr map map_int map_chr map_lgl map2
#' @import rlang
"_PACKAGE"

