# Generated by cpp11: do not edit by hand

escapeExamples <- function(x) {
  .Call(`_roxygen2_escapeExamples`, x)
}

findEndOfTag <- function(string, is_code, start) {
  .Call(`_roxygen2_findEndOfTag`, string, is_code, start)
}

rdComplete <- function(string, is_code) {
  .Call(`_roxygen2_rdComplete`, string, is_code)
}

leadingSpaces <- function(lines) {
  .Call(`_roxygen2_leadingSpaces`, lines)
}

tokenise_block <- function(lines, file, offset) {
  .Call(`_roxygen2_tokenise_block`, lines, file, offset)
}

find_includes <- function(path) {
  .Call(`_roxygen2_find_includes`, path)
}

wrapUsage <- function(string, width, indent) {
  .Call(`_roxygen2_wrapUsage`, string, width, indent)
}
