# exporting a call to :: produces re-exports documentation

    % Generated by roxygen2: do not edit by hand
    % Please edit documentation in ./<text>
    \docType{import}
    \name{reexports}
    \alias{reexports}
    \alias{auto_test}
    \title{Objects exported from other packages}
    \keyword{internal}
    \description{
    These objects are imported from other packages. Follow the links
    below to see their documentation.
    
    \describe{
      \item{testthat}{\code{\link[testthat]{auto_test}}}
    }}

