# format.rd_r6_inherited renders method list

    Code
      cat(format(inherited), sep = "\n")
    Output
      \if{html}{\out{<details open><summary>Inherited methods</summary>
      <ul>
        <li><code>pkg::A$foo()</code></li>
        <li><code>pkg::A$bar()</code></li>
      </ul>
      </details>}}

