# careful ordering

    Code
      out
    Output
      $foo1.Rd
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{foo1}
      \alias{foo1}
      \title{foo1}
      \usage{
      foo1()
      }
      \description{
      foo1
      }
      \seealso{
      Other a:
      \code{\link[=Foo3]{Foo3()}},
      \code{\link[=foo]{foo()}},
      \code{\link[=foo2]{foo2()}}
      }
      \concept{a}
      
      $foo2.Rd
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{foo2}
      \alias{foo2}
      \title{foo2}
      \usage{
      foo2()
      }
      \description{
      foo2
      }
      \seealso{
      Other a:
      \code{\link[=Foo3]{Foo3()}},
      \code{\link[=foo]{foo()}},
      \code{\link[=foo1]{foo1()}}
      }
      \concept{a}
      
      $Foo3.Rd
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{Foo3}
      \alias{Foo3}
      \title{Foo3}
      \usage{
      Foo3()
      }
      \description{
      Foo3
      }
      \seealso{
      Other a:
      \code{\link[=foo]{foo()}},
      \code{\link[=foo1]{foo1()}},
      \code{\link[=foo2]{foo2()}}
      }
      \concept{a}
      
      $foo.Rd
      % Generated by roxygen2: do not edit by hand
      % Please edit documentation in ./<text>
      \name{foo}
      \alias{foo}
      \title{foo}
      \usage{
      foo()
      }
      \description{
      foo
      }
      \seealso{
      Other a:
      \code{\link[=Foo3]{Foo3()}},
      \code{\link[=foo1]{foo1()}},
      \code{\link[=foo2]{foo2()}}
      }
      \concept{a}
      

