% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-link.R
\name{markdown-test}
\alias{markdown-test}
\title{Dummy page to test roxygen's markdown formatting}
\description{
Links are very tricky, so I'll put in some links here:
Link to a function: \code{\link[=roxygenize]{roxygenize()}}.
Link to an object: \link{roxygenize} (we just treat it like an object here.
}
\details{
Link to another package, function: \code{\link[desc:desc]{desc::desc()}}.
Link to another package, non-function: \link[desc:desc]{desc::desc}.

Link with link text: \link[=roxygenize]{this great function},
\code{\link{roxygenize}}, or \link[=roxygenize]{that great function}.

In another package: \link[desc:desc]{and this one}.

This is a table:\tabular{lr}{
   \strong{foo} \tab \strong{bar} \cr
   1 \tab 2 \cr
   100 \tab 200 \cr
}
}
\keyword{internal}
