% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tag-metadata.R
\name{tags_list}
\alias{tags_list}
\alias{tags_metadata}
\title{Access metadata about built-in or available tags}
\usage{
tags_list(built_in = TRUE)

tags_metadata()
}
\arguments{
\item{built_in}{Logical. Whether to restrict the result to built-in tags
(\code{TRUE}) or to extend it to available tags (\code{FALSE}).}
}
\description{
Access metadata about built-in or available tags
}
\seealso{
Other extending:
\code{\link[=load_options]{load_options()}},
\code{\link[=parse_package]{parse_package()}},
\code{\link[=rd_section]{rd_section()}},
\code{\link[=roc_proc_text]{roc_proc_text()}},
\code{\link[=roclet_find]{roclet_find()}},
\code{\link[=roxy_block]{roxy_block()}},
\code{\link[=roxy_tag]{roxy_tag()}},
\code{\link[=roxy_tag_rd]{roxy_tag_rd()}},
\code{\link{tag_parsers}}
}
\concept{extending}
