% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-code.R
\name{markdown_evaluate}
\alias{markdown_evaluate}
\title{Expand embedded inline code}
\usage{
markdown_evaluate(text)
}
\arguments{
\item{text}{Input text.}
}
\value{
Text with R code expanded.
A character vector of the same length as the input \code{text}.
}
\description{
For example this becomes two: 2.
Variables can be set and then reused, within the same
tag:
The value of \code{x} is 100.

We have access to the internal functions of the package, e.g.
since this is \emph{roxygen2}, we can refer to the internal \code{markdown}
function, and this is \code{TRUE}: TRUE.

To insert the name of the current package: roxygen2.

The \code{iris} data set has 5 columns:
\code{Sepal.Length}, \code{Sepal.Width}, \code{Petal.Length}, \code{Petal.Width}, \code{Species}.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# Code block demo
x + 1
#> [1] 101
}\if{html}{\out{</div>}}

Chunk options:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{names(mtcars)
nrow(mtcars)
#>  [1] "mpg"  "cyl"  "disp" "hp"   "drat" "wt"   "qsec" "vs"   "am"   "gear"
#> [11] "carb"
#> [1] 32
}\if{html}{\out{</div>}}

Plots:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{plot(1:10)
}\if{html}{\out{</div>}}

\figure{test-figure-1.png}

Alternative knitr engines:

\if{html}{\out{<div class="sourceCode default">}}\preformatted{```\{r\}
# comment
this <- 10
is <- this + 10
good <- this + is
}\if{html}{\out{</div>}}

Also see \code{vignette("rd-formatting")}.
}
\keyword{internal}
