> res[[n]]
% Generated by roxygen2: do not edit by hand
% Please edit documentation in ./roxygen-block-3.R
\name{A}
\alias{A}
\title{Class A}
\description{
Class A description.
}
\details{
Class A details
}
\examples{

## ------------------------------------------------
## Method `A$meth1`
## ------------------------------------------------

## Example for meth1

## ------------------------------------------------
## Method `A$meth2`
## ------------------------------------------------

## Example for meth2
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{field1}}{A field 1.}

\item{\code{field2}}{A field 2.}

\item{\code{field3}}{A field 3.}
}
\if{html}{\out{</div>}}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{active1}}{A binding 1.}

\item{\code{active2}}{A binding 2.}

\item{\code{active3}}{A binding 2.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-A-meth1}{\code{A$meth1()}}
\item \href{#method-A-meth2}{\code{A$meth2()}}
\item \href{#method-A-meth3}{\code{A$meth3()}}
\item \href{#method-A-clone}{\code{A$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-A-meth1"></a>}}
\if{latex}{\out{\hypertarget{method-A-meth1}{}}}
\subsection{Method \code{meth1()}}{
A method 1.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{A$meth1(Z)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Z}}{zzzzzzz}
}
\if{html}{\out{</div>}}
}
\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{## Example for meth1
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-A-meth2"></a>}}
\if{latex}{\out{\hypertarget{method-A-meth2}{}}}
\subsection{Method \code{meth2()}}{
Method 2 description.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{A$meth2(Z = 10, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Z}}{Overriding Z argument for meth2.}

\item{\code{...}}{Rest.}
}
\if{html}{\out{</div>}}
}
\subsection{Details}{
Method 2 details.
}

\subsection{Examples}{
\if{html}{\out{<div class="r example copy">}}
\preformatted{## Example for meth2
}
\if{html}{\out{</div>}}

}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-A-meth3"></a>}}
\if{latex}{\out{\hypertarget{method-A-meth3}{}}}
\subsection{Method \code{meth3()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{A$meth3(duplicate, missing)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{duplicate}}{This one is}

\item{\code{duplicate}}{Twice}
}
\if{html}{\out{</div>}}
}
\subsection{Details}{
Method 3 details.
}

\subsection{Returns}{
twice.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-A-clone"></a>}}
\if{latex}{\out{\hypertarget{method-A-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{A$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

