> res[[n]]
% Generated by roxygen2: do not edit by hand
% Please edit documentation in ./roxygen-block-3.R
\name{C}
\alias{C}
\title{Class C}
\description{
Class C Description.
}
\details{
Classs C details.
}
\section{Super classes}{
\code{roxygen2::A} -> \code{roxygen2::B} -> \code{C}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{nosuchfield}}{This will warn.}

\item{\code{field5}}{C field 5.}

\item{\code{duplicatefield}}{Multiple.}

\item{\code{duplicatefield}}{times.}
}
\if{html}{\out{</div>}}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{nosuchfield}}{This will warn.}

\item{\code{active2}}{C binding 2.}

\item{\code{active4}}{C binding 4.}

\item{\code{active6}}{C binding 6.}

\item{\code{duplicate_binding}}{Double.}

\item{\code{duplicate_binding}}{Double double.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-C-meth2}{\code{C$meth2()}}
\item \href{#method-C-meth5}{\code{C$meth5()}}
\item \href{#method-C-undocumented_method}{\code{C$undocumented_method()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="A" data-id="meth3"><a href='../../roxygen2/html/A.html#method-A-meth3'><code>roxygen2::A$meth3()</code></a></span></li>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="B" data-id="meth1"><a href='../../roxygen2/html/B.html#method-B-meth1'><code>roxygen2::B$meth1()</code></a></span></li>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="B" data-id="meth4"><a href='../../roxygen2/html/B.html#method-B-meth4'><code>roxygen2::B$meth4()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-C-meth2"></a>}}
\if{latex}{\out{\hypertarget{method-C-meth2}{}}}
\subsection{Method \code{meth2()}}{
C method 2.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{C$meth2(Z = 10, ...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Z}}{zzzzz}

\item{\code{...}}{etc}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-C-meth5"></a>}}
\if{latex}{\out{\hypertarget{method-C-meth5}{}}}
\subsection{Method \code{meth5()}}{
C method 5.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{C$meth5()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-C-undocumented_method"></a>}}
\if{latex}{\out{\hypertarget{method-C-undocumented_method}{}}}
\subsection{Method \code{undocumented_method()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{C$undocumented_method()}\if{html}{\out{</div>}}
}

}
}

