> res[[n]]
% Generated by roxygen2: do not edit by hand
% Please edit documentation in ./roxygen-block-3.R
\name{B}
\alias{B}
\title{Class B}
\description{
Class B Description.
}
\details{
Class B details.
}
\section{Super class}{
\code{roxygen2::A} -> \code{B}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{field1}}{B field 1.}

\item{\code{field4}}{B field 4.}
}
\if{html}{\out{</div>}}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{active1}}{B binding 1.}

\item{\code{active4}}{B binding 4.}

\item{\code{active5}}{B binding 5.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-B-meth1}{\code{B$meth1()}}
\item \href{#method-B-meth4}{\code{B$meth4()}}
\item \href{#method-B-clone}{\code{B$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="A" data-id="meth2"><a href='../../roxygen2/html/A.html#method-A-meth2'><code>roxygen2::A$meth2()</code></a></span></li>
<li><span class="pkg-link" data-pkg="roxygen2" data-topic="A" data-id="meth3"><a href='../../roxygen2/html/A.html#method-A-meth3'><code>roxygen2::A$meth3()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-B-meth1"></a>}}
\if{latex}{\out{\hypertarget{method-B-meth1}{}}}
\subsection{Method \code{meth1()}}{
B method 1.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{B$meth1(Z)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Z}}{Still zzzzzzzz.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-B-meth4"></a>}}
\if{latex}{\out{\hypertarget{method-B-meth4}{}}}
\subsection{Method \code{meth4()}}{
A method 4.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{B$meth4()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-B-clone"></a>}}
\if{latex}{\out{\hypertarget{method-B-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{B$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

